#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "bgclip.h"

#define IMG_W  640
#define IMG_H  480

#define RED    0xff0000
#define BLACK  0
#define WHITE  0xffffff


int img[IMG_W*IMG_H];
unsigned char img3[IMG_W*IMG_H*3];

void checker(int bg, int fg,int siz)
{
  int x,y;
  for(x=0;x<IMG_W;x++)
    for(y=0;y<IMG_H;y++)
      if ((x/siz+y/siz)%2==0)
        img[x+y*IMG_W]= fg;
      else
        img[x+y*IMG_W]= bg;
}

void rect(int fg, int l, int t, int r, int b)
{
  int x,y;
  for(x=l;x<IMG_W && x<=r;x++)
    for(y=t;y<IMG_H && y<=b;y++)
       img[x+y*IMG_W]= fg;
}


void horline(int fg, int x1, int x2, int y)
{
  int x;
  if (y<IMG_H && y>=0)
    for(x=x1;x<IMG_W && x<=x2;x++)
       img[x+y*IMG_W]= fg;
}

void verline(int fg, int y1, int y2, int x)
{
  int y;
  if (x<IMG_W && x>=0)
    for(y=y1;y<IMG_H && y<=y2;y++)
       img[x+y*IMG_W]= fg;
}

void brect(int fg, int bo, int l, int t, int r, int b)
{
  rect(fg, l,t,r,b);
  horline(bo, l,r, t);
  horline(bo, l,r, b);
  verline(bo, t,b, l);
  verline(bo, t,b, r);
}

void imgconvert()
{
  int i;
  for(i=0;i<IMG_W*IMG_H;i++)
  {
    img3[3*i+2]= img[i] & 0xff;
    img3[3*i+1]= (img[i] >> 8) & 0xff;
    img3[3*i+0]= (img[i] >>16) & 0xff;
  }
}

void write_image(char *fn)
{
  FILE *f;
  imgconvert();
  f= fopen(fn,"w");
  if (!f) { printf("can't open file %s\n", fn); return ; }
  fprintf(f,"P6\n%d %d\n255\n", IMG_W, IMG_H);
  fwrite(img3, 1, sizeof(img3), f);
  fclose(f);
}

int main(int argc,char **argv)
{
  int N;
  int i;
  int L,R,B,T;
  bgclip_t *C;
  int seed;

  N= atoi(argv[1]);
  if (argc>2) seed= atoi(argv[2]);
         else seed= time(NULL);
  printf("seed= %d\n", seed);
  srand(seed);
  checker(0xffff00,0, 5);
  C= bgclip_start(0, IMG_W/4, IMG_H/4, 
                     IMG_W-IMG_W/4, IMG_H-IMG_H/4);

  for(i=0;i<N;i++)
  {
     L= rand() % (IMG_W-1);
     R= rand() % (IMG_W/10) + L;
     T= rand() % (IMG_H-1);
     B= rand() % (IMG_H/10) + T ;
     brect(BLACK,RED, L,T, R, B);
     bgclip_remove(C, L,T,R,B);
     printf("%d %d\n", i, (int) C->len);
  } 

  N= 0;
    int W;
    while(bgclip_get(C, &L,&T,&R,&B))
    {
      W= 0xff - N;
      N++;
      W|= W<<16 | W<<8;
      rect(W, L,T,R,B);
    }

  write_image("out.ppm");
  return 0;
}
